@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 
-
-""" A Spotify API wrapper dedicated to people with a brain """
-
 import json
 import aiohttp
 import asyncio
@@ -176,15 +173,44 @@ class Spotify(object):
     items = []
     if ids != None:
       items = [self.get_id('track', i) for i in ids]
-    return await self.send_request('PUT', 'https://api.spotify.com/v1/me/tracks', ids=",".join(items))
+    return await self.send_request('PUT', 'https://api.spotify.com/v1/me/tracks', ids=','.join(items))
 
   async def unheart(self, ids):
     items = []
     if ids != None:
       items = [self.get_id('track', i) for i in ids]
-    return await self.send_request('DELETE', 'https://api.spotify.com/v1/me/tracks', ids=",".join(items))
+    return await self.send_request('DELETE', 'https://api.spotify.com/v1/me/tracks', ids=','.join(items))
 
+  async def search(self, query, limit=10, offset=0, type='track'):
+    if not type in ('track', 'artist', 'album', 'playlist'):
+      return False
+    return await self.send_request('GET', 'https://api.spotify.com/v1/search', q=query, limit=limit, offset=offset, type=type)
 
+  async def related_artists(self, id):
+    id = self.get_id('artist', id)
+    return await self.send_request('GET', 'https://api.spotify.com/v1/artists/%s/related-artists' % id)
+
+  async def album(self, id):
+    id = self.get_id('album', id)
+    return await self.send_request('GET', 'https://api.spotify.com/v1/albums/%s' % id)
+
+  async def albums(self, ids):
+    items = []
+    if ids != None:
+      items = [self.get_id('album', i) for i in ids]
+    return await self.send_request('GET', 'https://api.spotify.com/v1/albums', ids=','.join(items))
+
+  async def album_tracks(self, id, limit=50, offset=0):
+    id = self.get_id('album', id)
+    return await self.send_request('GET', 'https://api.spotify.com/v1/albums/%s/tracks' % id, limit=limit, offset=offset)
+
+  async def playlists(self, limit=50, offset=0):
+    return await self.send_request('GET', 'https://api.spotify.com/v1/me/playlists', limit=limit, offset=offset)
+  
+  async def playlist(self, id, fields=None, types=('track',)):
+    id = self.get_id('playlist', id)
+    return await self.send_request('GET', 'https://api.spotify.com/v1/me/playlists%s' % id, fields=fields, types=','.join(types))
+  
 
   async def queue(self, track, device=None):
     uri = self.get_uri('track', track)
@@ -199,6 +225,9 @@ class Spotify(object):
   async def playing_track(self):
     return await self.send_request('GET', 'https://api.spotify.com/v1/me/player/currently-playing')
 
+  async def current(self):
+    return await self.send_request('GET', 'https://api.spotify.com/v1/me/player')
+
   async def followed_artists(self, limit=10, after=None):
     return await self.send_request('GET', 'https://api.spotify.com/v1/me/following', type='artist', limit=limit, after=after)
 
@@ -220,3 +249,14 @@ class Spotify(object):
 
   async def saved(self, limit=10, offset=0):
     return await self.send_request('GET', 'https://api.spotify.com/v1/me/tracks', limit=limit, offset=offset)
+
+  async def devices(self):
+    return await self.send_request('GET', 'https://api.spotify.com/v1/me/player/devices')
+
+  async def swap_device(self, device, force=True):
+    payload = {
+      'device_ids': [device],
+      'play': force
+    }
+
+    return await self.send_request('PUT', 'https://api.spotify.com/v1/me/player', payload=payload)
